#include <stdint.h>
#include <stdbool.h>

/* NOTE: example is utilized by running the BLE Scanner application on a smartphone or tablet
 * BLE Scanner can be found here:
 * https://play.google.com/store/apps/details?id=com.macdom.ble.blescanner&hl=en
 *
 * The MCU will update the status of the a port and send it over BLE
 * The value of the port can be found in BLE scanner after connecting to the device
 * under "Custom service", first custom characteristic ( UUID of the characteristic ends with "703" )
 * You can also set the value of PORT over your smartphone/tablet, by writing to the characteristic
 * Make sure that the notifications are enabled in your BLE3 Scanner application
 */
 

sbit BLE3_RTS_MCU at GPIOD_IDR.B13;
sbit BLE3_CTS_MCU at GPIOD_ODR.B10;
sbit BLE3_RST     at  GPIOC_ODR.B2;


uint16_t porte_read = 0;

void main()
{

    GPIO_Digital_Input( &GPIOD_BASE, _GPIO_PINMASK_10 );
    GPIO_Digital_Output( &GPIOD_BASE, _GPIO_PINMASK_13 );
    GPIO_Digital_Output ( &GPIOC_BASE, _GPIO_PINMASK_2 );
    GPIO_Digital_Output(&GPIOE_BASE, _GPIO_PINMASK_ALL); // Set PORTE as digital output

    UART3_Init_Advanced( 115200, _UART_8_BIT_DATA,       // Initialize UART
                                _UART_NOPARITY,
                                _UART_ONE_STOPBIT,
                                &_GPIO_MODULE_USART3_PD89);
    delay_ms(500);

    BLE3_RTS_MCU = 0;            // set RTS to 0
    BLE3_RST = 1;                // pull reset pin high
    delay_ms(2000);
    
    Uart3_Write_Text("AT+UBTLN=");   // set the device name to "mikroE"
    Uart3_Write_Text("\"mikroE\"");
    Uart3_Write_Text("\r");

    delay_ms(2000);

    Uart3_Write_Text("ATO1\r");   // set BLE 3 into data mode, all bytes sent from UART will be sent over BLE
    delay_ms(1000);               // all bytes received from BLE will be sent to UART

    
    RXNEIE_USART3_CR1_bit = 1;           // setup UART interrupt
    NVIC_IntEnable( IVT_INT_USART3 );
    EnableInterrupts();

    while(1)
    {
         GPIOE_ODR = porte_read << 8;
         Uart3_Write(porte_read);        // send the updated value of the PORT over BLE
         delay_ms(1000);
         porte_read++;
    }

}

// if smartphone/tablet writes to BLE 3, update the port with the value sent from the smartphone/tablet

void LO_RX_ISR() iv IVT_INT_USART3 ics ICS_AUTO
{
    if( RXNE_USART3_SR_bit )
    {
        porte_read = USART3_DR;
    }
}